#include <windows.h>
#include <IntLib.h>
#include <CoProcLib.h>
#include <MapRegLib.h>


// NOTE: We want to interrupt on a GPIO located on Pin 48 of the Lower PlugBoard of the Colibri evaluation board (EXT_IO[0]).
// On the different Colibri Modules this corresponds to different GPIOs:
// On PXA270: GPIO35
// On PXA320: GPIO95
// On PXA300: GPIO97

#define GPIO_BASE	0x40E00000

typedef volatile struct 
{
    DWORD gplr0;	// Level Reg. Bank 0
    DWORD gplr1;	// Level Reg. Bank 1
    DWORD gplr2;	// Level Reg. Bank 2
    DWORD gpdr0;	// Direction Reg. Bank 0
    DWORD gpdr1;	// Direction Reg. Bank 1
    DWORD gpdr2;	// Direction Reg. Bank 2
    DWORD gpsr0;	// Output Set Reg. Bank 0
    DWORD gpsr1;	// Output Set Reg. Bank 1
    DWORD gpsr2;	// Output Set Reg. Bank 2
    DWORD gpcr0;	// Output Clr Reg. Bank 0
    DWORD gpcr1;	// Output Clr Reg. Bank 1
    DWORD gpcr2;	// Output Clr Reg. Bank 2
    DWORD grer0;	// Ris. Edge Detect Enable Reg. Bank 0
    DWORD grer1;	// Ris. Edge Detect Enable Reg. Bank 1
    DWORD grer2;	// Ris. Edge Detect Enable Reg. Bank 2
    DWORD gfer0;	// Fal. Edge Detect Enable Reg. Bank 0
    DWORD gfer1;	// Fal. Edge Detect Enable Reg. Bank 1
    DWORD gfer2;	// Fal. Edge Detect Enable Reg. Bank 2
    DWORD gedr0;	// Edge Detect Status Reg. Bank 0
    DWORD gedr1;	// Edge Detect Status Reg. Bank 1
    DWORD gedr2;	// Edge Detect Status Reg. Bank 2
    DWORD gafr0_l;	// Alt. Function Select Reg.[  0:15 ]
    DWORD gafr0_u;	// Alt. Function Select Reg.[ 16:31 ]
    DWORD gafr1_l;	// Alt. Function Select Reg.[ 32:47 ]
    DWORD gafr1_u;	// Alt. Function Select Reg.[ 48:63 ]
    DWORD gafr2_l;	// Alt. Function Select Reg.[ 64:79 ]
    DWORD gafr2_u;	// Alt. Function Select Reg.[ 80:95 ]
    DWORD gafr3_l;	// Alt. Function Select Reg.[ 96:111]
    DWORD gafr3_u;	// Alt. Function Select Reg.[112:120]
    DWORD res1[35];
    DWORD gplr3;	// Level Detect Reg. Bank 3
    DWORD res2[2];
    DWORD gpdr3;	// Data Direction Reg. Bank 3
    DWORD res3[2];
    DWORD gpsr3;	// Pin Output Set Reg. Bank 3
    DWORD res4[2];
    DWORD gpcr3;	// Pin Output Clr Reg. Bank 3
    DWORD res5[2];
    DWORD grer3;	// Ris. Edge Detect Enable Reg. Bank 3
    DWORD res6[2];
    DWORD gfer3;	// Fal. Edge Detect Enable Reg. Bank 3
    DWORD res7[2];
    DWORD gedr3;	// Edge Detect Status Reg. Bank 3

} GPIOREG;


int WINAPI WinMain(HINSTANCE hInstance,      // handle to current instance
                   HINSTANCE hPrevInstance,  // handle to previous instance
                   LPWSTR lpCmdLine,          // pointer to command line
                   int nCmdShow)             // show state of window
{
	HANDLE	hEvent;
	DWORD	dwEdge, dwGpio;
	DWORD	dwIrq, dwSysIntr;
	GPIOREG*	pGPIORegs;
	DWORD	ProcID=getPROCID();
 
	//************************************
	// GPIO Specific Section

	pGPIORegs=MapRegister(GPIO_BASE);
	switch(ProcID){
	
		case TYPE_PXA270:
				dwGpio=35;
				pGPIORegs->gpdr1&=~(1<<(35-32)); // Set GPIO35 as input
			break;

		case TYPE_PXA320:
				dwGpio=95;
				pGPIORegs->gpdr2&=~(1<<(95-64)); // Set GPIO95 as input
			break;
		
		case TYPE_PXA310:
		case TYPE_PXA300:
				dwGpio=97;
				pGPIORegs->gpdr3&=~(1<<(97-96)); // Set GPIO97 as input
		break;
	}
	UnMapRegister((void*)pGPIORegs);

	//In case that you own the GPIOLib you can just call
	//SetGPIODir(dwGpio, DIR_IN); to set the direction


	dwIrq=GetGPIOIrq(dwGpio);
	if(!dwIrq) goto leave_error; 

	// Set Edge to trigger (Rising, Falling or both)
	//dwEdge=GPIO_EDGE_RISING | GPIO_EDGE_FALLING; // (rising & falling edge)
	dwEdge=GPIO_EDGE_FALLING;
	//dwEdge=GPIO_EDGE_RISING;	
	if(!SetGPIOIrqEdge(dwGpio, dwEdge)) goto leave_error;
 

	//************************************
	// General Interrupt Handling Section	

	// Create an Event to wait on
	hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	if ( hEvent == NULL) goto leave_error;

	// Get the SYSINTR that corresponds to dwIrq (non GPIO Irqs corresond to the bit numbers of the PXA27x Interrupt resisters)
	dwSysIntr=RequestSysInterrupt(dwIrq);
	if (!dwSysIntr) goto leave_error;

	// Link our Event with the SYSINTR
	if ( !InterruptInitializeCompat(dwSysIntr, hEvent, NULL, 0) ) goto leave_error;

	CeSetThreadPriority(GetCurrentThread(), 200);

	// NOTE: Normally this will be done in a separate Thread
	while(1)
	{
		// Wait for Event (Interrupt)
		if (WaitForSingleObject(hEvent, INFINITE) == WAIT_OBJECT_0)
		{
			if(MessageBox(NULL, L"Interrupt Event1 Detected, continue Waiting?", L"Interrupt", MB_YESNO) == IDNO) break;
			// NOTE: It's possible that we get the same interrupt twice if the signal is not debounced
			InterruptDoneCompat(dwSysIntr);
		}
	}

	InterruptDisableCompat(dwSysIntr);

	
	if (!ReleaseSysIntr(dwSysIntr)) goto leave_error;


leave_error:

    return(TRUE);


}

