/*
Toradex MapRegLib.h

Header file for the free available MapReg Lib

See also the free sample RegAccess

Requires:
- CoProcLib.lib
- KernelCallbackLib.lib

***13.01.09
Add
-OALPAtoVA

***26.09.08
Add 
-AllocPhysMemCompat
-FreePhysMemCompat

*/

#ifndef __MAPREG_H__
#define __MAPREG_H__


#ifdef __cplusplus
extern "C" {
#endif


#define MRL_VER_MAJ			1
#define MRL_VER_MIN			1
#define MRL_VER_BUILD       0


#define PAGE_SIZE     4096


/*
Sturct for phisical and virtuall addresses.
(e.g usefull for DMA opterations)
*/
typedef struct {
	DWORD pa;
	DWORD va;
} MEM_T;


/*
Map Registers at a phisical address
(does map areas up to the PAGE_SIZE)
pa: Phisical address
return: pointer to the virtual address
*/
void* MapRegister(DWORD pa);

/*
Unmap Registers
pRegs: pointer to virual address
*/
void UnMapRegister(volatile void* pRegs);

/*
Same than MapRegister() but you can also map larger sections
pa: phisical address
size: Size to mape in bytes
*/
void* MapMemory(DWORD pa, DWORD size);
#define UnMapMemory UnMapRegister


// Common Windows API not defined in EVC++
LPVOID AllocPhysMem(DWORD cbSize, DWORD fdwProtect, DWORD dwAlignmentMask, DWORD dwFlags, PULONG pPhysicalAddress);
BOOL FreePhysMem(LPVOID lpvAddress);
BOOL VirtualCopy(LPVOID lpvDest, LPVOID lpvSrc, DWORD cbSize, DWORD fdwProtect);
BOOL VirtualSetAttributes(LPVOID lpvAddress, DWORD cbSize, DWORD dwNewFlags, DWORD dwMask, LPDWORD lpdwOldFlags);
LPVOID CreateStaticMapping(DWORD dwPhysBase, DWORD dwSize);

//Win CE 6.0 and 5.0 compatible version of AllocPhysMem()
LPVOID AllocPhysMemCompat(DWORD cbSize, DWORD fdwProtect, DWORD dwAlignmentMask, DWORD dwFlags, PULONG pPhysicalAddress);

//Win CE 6.0 and 5.0 compatible version of FreePhysMem()
BOOL FreePhysMemCompat(LPVOID lpvAddress);

//Win CE 6.0 and 5.0 compatible version of VirtualCopy()
BOOL VirtualCopyCompat(LPVOID lpvDest, LPVOID lpvSrc, DWORD cbSize, DWORD fdwProtect);

//Win CE 6.0 and 5.0 compatible version of LockPages()
BOOL LockPagesCompat(LPVOID lpvAddress, DWORD cbSize, PDWORD pPFNs, int fOptions);

//Win CE 6.0 and 5.0 compatible version of VirtualSetAttributes()
BOOL VirtualSetAttributesCompat(LPVOID lpvAddress, DWORD cbSize, DWORD dwNewFlags, DWORD dwMask, LPDWORD lpdwOldFlags);


void* OALPAtoVA(UINT32 pa, BOOL cached);


/*
Description:      Returns the library Version
Parameter:
Out-Parameter:
- pVerMaj:          Returns the major version number. Set this parameter to NULL if not required.
- pVerMin:          Returns the minor version number. Set this parameter to NULL if not required.
- pBuild:           Returns the build number.         Set this parameter to NULL if not required.
Return Value:		No return value.
*/
void MRLGetLibVersion(DWORD *pVerMaj, DWORD *pVerMin, DWORD *pBuild);


#ifdef __cplusplus
}
#endif

#endif

